@@ -9,6 +9,7 @@
 
 #include <afs/param.h>
 #include <afs/stds.h>
+#include <afs/cellconfig.h>
 
 #ifndef DJGPP
 #include <windows.h>
@@ -22,21 +23,20 @@
 #include <string.h>
 
 #include "cm_config.h"
+#include <WINNT\afssw.h>
 #ifdef AFS_AFSDB_ENV
-#if !defined(DJGPP) && defined(_WIN32_WINNT) && _WIN32_WINNT >= 0x500
-#include <windns.h>
-#define DNSAPI_ENV
-#else
 #include "cm_dns.h"
-#endif
 #include <afs/afsint.h>
 #endif
 
 char AFSConfigKeyName[] =
 	"SYSTEM\\CurrentControlSet\\Services\\TransarcAFSDaemon\\Parameters";
+char AFSLocalMachineKeyName[] = "SOFTWARE\\OpenAFS\\Client";
 
 /* TODO: these should be pulled in from dirpath.h */
+#if !defined(DJGPP) && !defined(AFS_WIN95_ENV)
 #define AFS_THISCELL "ThisCell"
+#endif
 #define AFS_CELLSERVDB_UNIX "CellServDB"
 #define AFS_CELLSERVDB_NT "afsdcell.ini"
 #ifndef AFSDIR_CLIENT_ETC_DIRPATH
@@ -49,18 +49,20 @@ extern char cm_confDir[];
 extern int errno;
 #endif /* DJGPP */
 #else
-#define AFS_CELLSERVDB AFS_CELLSERVDB_NT
+#define AFS_CELLSERVDB AFS_CELLSERVDB_UNIX
 #endif /* DJGPP || WIN95 */
 
-DWORD TraceOption=1;
+static DWORD TraceOption = 0;
 
-#define TRACE_OPTION_EVENT 1
-#define ISLOGONTRACE(v) ( ((v) & TRACE_OPTION_EVENT)==TRACE_OPTION_EVENT)
+/* This really needs to be initialized at DLL Init */
+#define TRACE_OPTION_EVENT 4
+
+#define ISCONFIGTRACE(v) ( ((v) & TRACE_OPTION_EVENT)==TRACE_OPTION_EVENT)
 
 void DebugEvent0_local(char *a) 
 {
 	HANDLE h; char *ptbuf[1];
-	if (!ISLOGONTRACE(TraceOption))
+	if (!ISCONFIGTRACE(TraceOption))
 		return;
 	h = RegisterEventSource(NULL, a);
 	ptbuf[0] = a;
@@ -74,7 +76,7 @@ void DebugEvent_local(char *a,char *b,...)
 {
 	HANDLE h; char *ptbuf[1],buf[MAXBUF_+1];
 	va_list marker;
-	if (!ISLOGONTRACE(TraceOption))
+	if (!ISCONFIGTRACE(TraceOption))
 		return;
 	h = RegisterEventSource(NULL, a);
 	va_start(marker,b);
@@ -85,6 +87,45 @@ void DebugEvent_local(char *a,char *b,...)
 	va_end(marker);
 }
 
+#define REG_CLIENT_PARMS_KEY            TEXT("SYSTEM\\CurrentControlSet\\Services\\TransarcAFSDaemon\\Parameters")
+#define REG_CLIENT_TRACE_OPTION_PARM	TEXT("TraceOption")
+
+#ifdef COMMENT
+BOOL WINAPI DllMain (HANDLE hModule, DWORD fdwReason, LPVOID lpReserved)
+{
+    switch (fdwReason)
+    {
+    case DLL_PROCESS_ATTACH: {
+        DWORD LSPtype, LSPsize;
+        HKEY NPKey;
+
+        (void) RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_CLIENT_PARMS_KEY,
+                             0, KEY_QUERY_VALUE, &NPKey);
+        LSPsize=sizeof(TraceOption);
+        RegQueryValueEx(NPKey, REG_CLIENT_TRACE_OPTION_PARM, NULL,
+                        &LSPtype, (LPBYTE)&TraceOption, &LSPsize);
+
+        RegCloseKey (NPKey);
+        break;
+    }
+
+    case DLL_THREAD_ATTACH:
+        break;
+
+    case DLL_THREAD_DETACH:
+        break;
+
+    case DLL_PROCESS_DETACH:
+        break;
+
+    default:
+        return FALSE;
+    }
+
+    return TRUE;   // successful DLL_PROCESS_ATTACH
+}
+#endif /* COMMENT */
+
 static long cm_ParsePair(char *lineBufferp, char *leftp, char *rightp)
 {
 	char *tp;
@@ -151,41 +192,21 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
 	cm_configProc_t *procp, void *rockp)
 {
 	char wdir[257];
-    int tlen;
-    FILE *tfilep, *bestp, *tempp;
+    FILE *tfilep = NULL, *bestp, *tempp;
     char *tp;
     char lineBuffer[257];
     struct hostent *thp;
     char *valuep;
     struct sockaddr_in vlSockAddr;
     int inRightCell;
-    int foundCell;
+    int foundCell = 0;
     long code;
 	int tracking = 1, partial = 0;
 #if defined(DJGPP) || defined(AFS_WIN95_ENV)
-	long ip_addr;
-    int c1, c2, c3, c4;
-    char aname[241];
     char *afsconf_path;
 #endif
 
-	foundCell = 0;
-
-#if !defined(DJGPP)
-	code = GetWindowsDirectory(wdir, sizeof(wdir));
-    if (code == 0 || code > sizeof(wdir))
-        return -1;
-
-	/* add trailing backslash, if required */
-    tlen = strlen(wdir);
-    if (wdir[tlen-1] != '\\') strcat(wdir, "\\");
-#else
-    strcpy(wdir, cm_confDir);
-    strcat(wdir,"/");
-#endif /* !DJGPP */
-        
-    strcat(wdir, AFS_CELLSERVDB);
-
+    cm_GetCellServDB(wdir);
     tfilep = fopen(wdir, "r");
 
 #if defined(DJGPP) || defined(AFS_WIN95_ENV)
@@ -200,7 +221,7 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
         else
             strcpy(wdir, afsconf_path);
         strcat(wdir, "/");
-        strcat(wdir, AFS_CELLSERVDB_UNIX);
+        strcat(wdir, AFS_CELLSERVDB);
         /*fprintf(stderr, "opening cellservdb file %s\n", wdir);*/
         tfilep = fopen(wdir, "r");
         if (!tfilep) return -2;
@@ -213,7 +234,9 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
 
 	bestp = fopen(wdir, "r");
     
+#ifdef DEBUG
     DebugEvent_local("AFS- cm_searchfile fopen", "Handle[%x], wdir[%s]", bestp, wdir);
+#endif
 
 	/* have we seen the cell line for the guy we're looking for? */
 	inRightCell = 0;
@@ -269,8 +292,10 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
 					strcpy(newCellNamep, lineBuffer+1);
                 inRightCell = 1;
 				tracking = 0;
+#ifdef DEBUG
                 DebugEvent_local("AFS- cm_searchfile is cell", "inRightCell[%x], linebuffer[%s]", 
                                  inRightCell, lineBuffer);
+#endif
 			}
 			else if (strnicmp(lineBuffer+1, cellNamep,
                                strlen(cellNamep)) == 0) {
@@ -310,13 +335,14 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
 				/* add the server to the VLDB list */
                 WSASetLastError(0);
                 thp = gethostbyname(valuep);
+#ifdef DEBUG
                 {
-                    int iErr = 0;
-                    iErr = WSAGetLastError();
+                    int iErr = WSAGetLastError();
                     DebugEvent_local("AFS- cm_searchfile inRightCell", 
                                      "thp[%x], valuep[%s], WSAGetLastError[%d]", 
                                      thp, valuep, iErr);
                 }
+#endif
                 if (thp) {
 					memcpy(&vlSockAddr.sin_addr.s_addr, thp->h_addr,
                             sizeof(long));
@@ -327,23 +353,32 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
                     foundCell = 1;
 				}
 #else
-                /* For DJGPP, we will read IP address instead
-                of name/comment field */
-                code = sscanf(lineBuffer, "%d.%d.%d.%d #%s",
-                               &c1, &c2, &c3, &c4, aname);
-                tp = (char *) &ip_addr;
-                *tp++ = c1;
-                *tp++ = c2;
-                *tp++ = c3;
-                *tp++ = c4;
-                memcpy(&vlSockAddr.sin_addr.s_addr, &ip_addr,
-                        sizeof(long));
-                vlSockAddr.sin_family = AF_INET;
-                /* sin_port supplied by connection code */
-                if (procp)
-                    (*procp)(rockp, &vlSockAddr, valuep);
-                foundCell = 1;
+                thp = 0;
 #endif /* !DJGPP */
+                if (!thp) {
+                    long ip_addr;
+					int c1, c2, c3, c4;
+					char aname[241] = "";                    
+                    
+                    /* Since there is no gethostbyname() data 
+                     * available we will read the IP address
+                     * stored in the CellServDB file
+                     */
+                    code = sscanf(lineBuffer, "%d.%d.%d.%d #%s",
+                                   &c1, &c2, &c3, &c4, aname);
+                    tp = (char *) &ip_addr;
+                    *tp++ = c1;
+                    *tp++ = c2;
+                    *tp++ = c3;
+                    *tp++ = c4;
+                    memcpy(&vlSockAddr.sin_addr.s_addr, &ip_addr,
+                            sizeof(long));
+                    vlSockAddr.sin_family = AF_INET;
+                    /* sin_port supplied by connection code */
+                    if (procp)
+                        (*procp)(rockp, &vlSockAddr, valuep);
+                    foundCell = 1;
+                }
             }
         }	/* a vldb line */
     }		/* while loop processing all lines */
@@ -356,23 +391,25 @@ long cm_SearchCellByDNS(char *cellNamep, char *newCellNamep, int *ttl,
                cm_configProc_t *procp, void *rockp)
 {
 #ifdef AFS_AFSDB_ENV
-#ifndef DNSAPI_ENV
     int rc;
-    int cellHosts[AFSMAXCELLHOSTS];
+    int  cellHostAddrs[AFSMAXCELLHOSTS];
+    char cellHostNames[AFSMAXCELLHOSTS][MAXHOSTCHARS];
     int numServers;
     int i;
     struct sockaddr_in vlSockAddr;
 
+#ifdef DEBUG
     DebugEvent_local("AFS SearchCellDNS-","Doing search for [%s]", cellNamep);
-    rc = getAFSServer(cellNamep, cellHosts, &numServers, ttl);
+#endif
+    rc = getAFSServer(cellNamep, cellHostAddrs, cellHostNames, &numServers, ttl);
     if (rc == 0 && numServers > 0) {     /* found the cell */
         for (i = 0; i < numServers; i++) {
-            memcpy(&vlSockAddr.sin_addr.s_addr, &cellHosts[i],
+            memcpy(&vlSockAddr.sin_addr.s_addr, &cellHostAddrs[i],
                    sizeof(long));
            vlSockAddr.sin_family = AF_INET;
            /* sin_port supplied by connection code */
            if (procp)
-          (*procp)(rockp, &vlSockAddr, NULL);
+          (*procp)(rockp, &vlSockAddr, cellHostNames[i]);
            if(newCellNamep)
           strcpy(newCellNamep,cellNamep);
         }
@@ -380,102 +417,50 @@ long cm_SearchCellByDNS(char *cellNamep, char *newCellNamep, int *ttl,
     }
     else
        return -1;  /* not found */
-#else /* DNSAPI_ENV */
-	PDNS_RECORD pDnsCell, pDnsIter, pDnsVol,pDnsVolIter, pDnsCIter;
-	LPSTR vlServers[AFSMAXCELLHOSTS];
-	IP4_ADDRESS vlAddrs[AFSMAXCELLHOSTS];
-	WORD nvlServers;
-	DWORD wttl, i;
-	BOOL success;
-    struct sockaddr_in vlSockAddr;
-
-	success = FALSE;
-
-    DebugEvent_local("AFS SearchCellDNS-","Doing search for [%s]", cellNamep);
-
-    /* query the AFSDB records of cell */
-	if(DnsQuery_A(cellNamep, DNS_TYPE_AFSDB, DNS_QUERY_STANDARD, NULL, &pDnsCell, NULL) == ERROR_SUCCESS) {
-
-		memset((void*) &vlSockAddr, 0, sizeof(vlSockAddr));
-		
-		nvlServers = 0; wttl = 0;
-
-		/* go through the returned records */
-		for(pDnsIter = pDnsCell;pDnsIter; pDnsIter = pDnsIter->pNext) {
-			/* if we find an AFSDB record with Preference set to 1, we found a volserver */
-			if(pDnsIter->wType == DNS_TYPE_AFSDB && pDnsIter->Data.Afsdb.wPreference == 1) {
-				vlServers[nvlServers++] = pDnsIter->Data.Afsdb.pNameExchange;
-				if(!wttl) wttl = pDnsIter->dwTtl;
-				if(nvlServers == AFSMAXCELLHOSTS) break;
-			}
-		}
-
-		for(i=0;i<nvlServers;i++) vlAddrs[i] = 0;
-
-		/* now check if there are any A records in the results */
-		for(pDnsIter = pDnsCell; pDnsIter; pDnsIter = pDnsIter->pNext) {
-			if(pDnsIter->wType == DNS_TYPE_A)
-				/* check if its for one of the volservers */
-				for(i=0;i<nvlServers;i++)
-					if(stricmp(pDnsIter->pName, vlServers[i]) == 0)
-						vlAddrs[i] = pDnsIter->Data.A.IpAddress;
-		}
-
-		for(i=0;i<nvlServers;i++) {
-			/* if we don't have an IP yet, then we should try resolving the volserver hostname
-			   in a separate query. */
-			if(!vlAddrs[i]) {
-				if(DnsQuery_A(vlServers[i], DNS_TYPE_A, DNS_QUERY_STANDARD, NULL, &pDnsVol, NULL) == ERROR_SUCCESS) {
-					for(pDnsVolIter = pDnsVol; pDnsVolIter; pDnsVolIter=pDnsVolIter->pNext) {
-						/* if we get an A record, keep it */
-						if(pDnsVolIter->wType == DNS_TYPE_A && stricmp(vlServers[i], pDnsVolIter->pName)==0) {
-							vlAddrs[i] = pDnsVolIter->Data.A.IpAddress;
-							break;
-						}
-						/* if we get a CNAME, look for a corresponding A record */
-						if(pDnsVolIter->wType == DNS_TYPE_CNAME && stricmp(vlServers[i], pDnsVolIter->pName)==0) {
-							for(pDnsCIter=pDnsVolIter; pDnsCIter; pDnsCIter=pDnsCIter->pNext) {
-								if(pDnsCIter->wType == DNS_TYPE_A && stricmp(pDnsVolIter->Data.CNAME.pNameHost, pDnsCIter->pName)==0) {
-									vlAddrs[i] = pDnsCIter->Data.A.IpAddress;
-									break;
-								}
-							}
-							if(vlAddrs[i]) break;
-							/* TODO: if the additional section is missing, then do another lookup for the CNAME */
-						}
-					}
-					/* we are done with the volserver lookup */
-					DnsRecordListFree(pDnsVol, DnsFreeRecordListDeep);
-				}
-			}
-
-			/* if we found a volserver, then add it */
-			if(vlAddrs[i]) {
-				vlSockAddr.sin_family = AF_INET;
-				vlSockAddr.sin_addr.s_addr = vlAddrs[i];
-				if(procp)
-					(*procp)(rockp, &vlSockAddr, vlServers[i]);
-				success = TRUE;
-			}
-		}
-
-		DnsRecordListFree(pDnsCell, DnsFreeRecordListDeep);
-	}
-
-	if(!success) return -1;
-	else {
-		strcpy(newCellNamep, cellNamep);
-		if(ttl) *ttl = (int) wttl;
-		return 0;
-	}
-
-#endif /* DNSAPI_ENV */
 #else
 	return -1;  /* not found */
 #endif /* AFS_AFSDB_ENV */
 }
 
 #if !defined(DJGPP) && !defined(AFS_WIN95_ENV)
+/* look up the CellServDBDir's name in the Registry 
+ * or use the Client Dirpath value to produce a CellServDB 
+ * filename
+ */
+long cm_GetCellServDB(char *cellNamep)
+{
+#if !defined(DJGPP)
+	DWORD code, dummyLen;
+	HKEY parmKey;
+    int tlen;
+
+	code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSLocalMachineKeyName,
+				0, KEY_QUERY_VALUE, &parmKey);
+	if (code != ERROR_SUCCESS)
+        goto dirpath;
+
+	dummyLen = 256;
+	code = RegQueryValueEx(parmKey, "CellServDBDir", NULL, NULL,
+				cellNamep, &dummyLen);
+	RegCloseKey (parmKey);
+
+  dirpath:
+	if (code != ERROR_SUCCESS || cellNamep[0] == 0)
+        strcpy(cellNamep, AFSDIR_CLIENT_ETC_DIRPATH);
+
+    /* add trailing backslash, if required */
+    tlen = strlen(cellNamep);
+    if (cellNamep[tlen-1] != '\\') 
+        strcat(cellNamep, "\\");
+#else
+    strcpy(cellNamep, cm_confDir);
+    strcat(cellNamep,"/");
+#endif /* !DJGPP */
+        
+    strcat(cellNamep, AFS_CELLSERVDB);
+	return 0;
+}
+
 /* look up the root cell's name in the Registry */
 long cm_GetRootCellName(char *cellNamep)
 {
@@ -537,14 +522,11 @@ long cm_GetRootCellName(char *cellNamep)
 cm_configFile_t *cm_CommonOpen(char *namep, char *rwp)
 {
 	char wdir[256];
-    long code;
     long tlen;
     FILE *tfilep;
 
 #if !defined(DJGPP) && !defined(AFS_WIN95_ENV)
-	code = GetWindowsDirectory(wdir, sizeof(wdir));
-        if (code == 0 || code > sizeof(wdir)) 
-            return 0;
+    strcpy(wdir, AFSDIR_CLIENT_ETC_DIRPATH);
         
 	/* add trailing backslash, if required */
         tlen = strlen(wdir);
@@ -714,75 +696,71 @@ long cm_CloseCellFile(cm_configFile_t *filep)
 	closeCode = fclose((FILE *)filep);
 
 #if !defined(DJGPP) && !defined(AFS_WIN95_ENV)
-	code = GetWindowsDirectory(wdir, sizeof(wdir));
-        if (code == 0 || code > sizeof(wdir)) 
-	    return closeCode;
+    strcpy(wdir, AFSDIR_CLIENT_ETC_DIRPATH);
         
 	/* add trailing backslash, if required */
-        tlen = strlen(wdir);
-        if (wdir[tlen-1] != '\\') strcat(wdir, "\\");
+    tlen = strlen(wdir);
+    if (wdir[tlen-1] != '\\') strcat(wdir, "\\");
 #else
 #ifdef DJGPP
-        strcpy(wdir,cm_confDir);
+    strcpy(wdir,cm_confDir);
 #else
-        afsconf_path = getenv("AFSCONF");
-        if (!afsconf_path)
-          strcpy(wdir, AFSDIR_CLIENT_ETC_DIRPATH);
-        else
-          strcpy(wdir, afsconf_path);
+    afsconf_path = getenv("AFSCONF");
+    if (!afsconf_path)
+        strcpy(wdir, AFSDIR_CLIENT_ETC_DIRPATH);
+    else
+        strcpy(wdir, afsconf_path);
 #endif /* !DJGPP */
-        strcat(wdir,"/");
+    strcat(wdir,"/");
 #endif /* DJGPP || WIN95 */
 
-        strcpy(sdir, wdir);
+    strcpy(sdir, wdir);
 
 	if (closeCode != 0) {
 		/* something went wrong, preserve original database */
-                strcat(wdir, "afsdcel2.ini");
-                unlink(wdir);
-                return closeCode;
-        }
+        strcat(wdir, "afsdcel2.ini");
+        unlink(wdir);
+        return closeCode;
+    }
 
-        strcat(wdir, AFS_CELLSERVDB);
-        strcat(sdir, "afsdcel2.ini");	/* new file */
-        
-        unlink(wdir);			/* delete old file */
-        
-        code = rename(sdir, wdir);	/* do the rename */
-        
-        if (code) 
-	  code = errno;
-        
-        return code;
-}
+    strcat(wdir, AFS_CELLSERVDB);
+    strcat(sdir, "afsdcel2.ini");	/* new file */
+
+    unlink(wdir);			/* delete old file */
+
+    code = rename(sdir, wdir);	/* do the rename */
+
+    if (code) 
+        code = errno;
+
+    return code;
+}   
 
 void cm_GetConfigDir(char *dir)
 {
-	char wdir[256];
-    int code;
+    char wdir[256];
     int tlen;
 #ifdef AFS_WIN95_ENV
     char *afsconf_path;
 #endif
 
 #if !defined(DJGPP) && !defined(AFS_WIN95_ENV)
-	code = GetWindowsDirectory(wdir, sizeof(wdir));
-      if (code == 0 || code > sizeof(wdir)) wdir[0] = 0;
+    strcpy(wdir, AFSDIR_CLIENT_ETC_DIRPATH);
         
 	/* add trailing backslash, if required */
-        tlen = strlen(wdir);
-        if (wdir[tlen-1] != '\\') strcat(wdir, "\\");
+    tlen = strlen(wdir);
+    if (wdir[tlen-1] != '\\') strcat(wdir, "\\");
 #else
 #ifdef DJGPP
-        strcpy(wdir,cm_confDir);
+    strcpy(wdir,cm_confDir);
 #else
-        afsconf_path = getenv("AFSCONF");
-        if (!afsconf_path)
-          strcpy(wdir, AFSDIR_CLIENT_ETC_DIRPATH);
-        else
-          strcpy(wdir, afsconf_path);
+    afsconf_path = getenv("AFSCONF");
+    if (!afsconf_path)
+        strcpy(wdir, AFSDIR_CLIENT_ETC_DIRPATH);
+    else
+        strcpy(wdir, afsconf_path);
 #endif /* !DJGPP */
-        strcat(wdir,"\\");
+    strcat(wdir,"\\");
 #endif /* DJGPP || WIN95 */
-        strcpy(dir, wdir);
+    strcpy(dir, wdir);
 }
