@@ -583,6 +583,8 @@ char *AclToString(acl)
     return mydata;
 }
 
+#define AFSCLIENT_ADMIN_GROUPNAME "AFS Client Admins"
+
 BOOL IsAdmin (void)
 {
     static BOOL fAdmin = FALSE;
@@ -590,57 +592,127 @@ BOOL IsAdmin (void)
 
     if (!fTested)
     {
-        /* Obtain the SID for BUILTIN\Administrators. If this is Windows NT,
-         * expect this call to succeed; if it does not, we can presume that
-         * it's not NT and therefore the user always has administrative
-         * privileges.
+        /* Obtain the SID for the AFS client admin group.  If the group does
+         * not exist, then assume we have AFS client admin privileges.
          */
         PSID psidAdmin = NULL;
-        SID_IDENTIFIER_AUTHORITY auth = SECURITY_NT_AUTHORITY;
+        DWORD dwSize, dwSize2;
+        char pszAdminGroup[ MAX_COMPUTERNAME_LENGTH + sizeof(AFSCLIENT_ADMIN_GROUPNAME) + 2 ];
+        char *pszRefDomain = NULL;
+        SID_NAME_USE snu = SidTypeGroup;
 
-        fTested = TRUE;
+        dwSize = sizeof(pszAdminGroup);
 
-        if (!AllocateAndInitializeSid (&auth, 2, SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS, 0, 0, 0, 0, 0, 0, &psidAdmin))
+        if (!GetComputerName(pszAdminGroup, &dwSize)) {
+            /* Can't get computer name.  We return false in this case.
+               Retain fAdmin and fTested. This shouldn't happen.*/
+            return FALSE;
+        }
+
+        dwSize = 0;
+        dwSize2 = 0;
+
+        strcat(pszAdminGroup,"\\");
+        strcat(pszAdminGroup, AFSCLIENT_ADMIN_GROUPNAME);
+
+        LookupAccountName(NULL, pszAdminGroup, NULL, &dwSize, NULL, &dwSize2, &snu);
+        /* that should always fail. */
+
+        if (GetLastError() != ERROR_INSUFFICIENT_BUFFER) {
+            /* if we can't find the group, then we allow the operation */
             fAdmin = TRUE;
-        else
-        {
+            return TRUE;
+        }
+
+        if (dwSize == 0 || dwSize2 == 0) {
+            /* Paranoia */
+            fAdmin = TRUE;
+            return TRUE;
+        }
+
+        psidAdmin = (PSID)malloc(dwSize); memset(psidAdmin,0,dwSize);
+        pszRefDomain = (char *)malloc(dwSize2);
+
+        if (!LookupAccountName(NULL, pszAdminGroup, psidAdmin, &dwSize, pszRefDomain, &dwSize2, &snu)) {
+            /* We can't lookup the group now even though we looked it up earlier.  
+               Could this happen? */
+            fAdmin = TRUE;
+        } else {
             /* Then open our current ProcessToken */
             HANDLE hToken;
 
             if (OpenProcessToken (GetCurrentProcess(), TOKEN_QUERY, &hToken))
             {
-                /* We'll have to allocate a chunk of memory to store the list of
-                 * groups to which this user belongs; find out how much memory
-                 * we'll need.
-                 */
-                DWORD dwSize = 0;
-                PTOKEN_GROUPS pGroups;
-                
-                GetTokenInformation (hToken, TokenGroups, NULL, dwSize, &dwSize);
-            
-                pGroups = (PTOKEN_GROUPS)malloc(dwSize);
-                
-                /* Allocate that buffer, and read in the list of groups. */
-                if (GetTokenInformation (hToken, TokenGroups, pGroups, dwSize, &dwSize))
-                {
-                    /* Look through the list of group SIDs and see if any of them
-                     * matches the Administrator group SID.
+
+                if (!CheckTokenMembership(hToken, psidAdmin, &fAdmin)) {
+                    /* We'll have to allocate a chunk of memory to store the list of
+                     * groups to which this user belongs; find out how much memory
+                     * we'll need.
                      */
-                    size_t iGroup = 0;
-                    for (; (!fAdmin) && (iGroup < pGroups->GroupCount); ++iGroup)
+                    DWORD dwSize = 0;
+                    PTOKEN_GROUPS pGroups;
+
+                    GetTokenInformation (hToken, TokenGroups, NULL, dwSize, &dwSize);
+
+                    pGroups = (PTOKEN_GROUPS)malloc(dwSize);
+
+                    /* Allocate that buffer, and read in the list of groups. */
+                    if (GetTokenInformation (hToken, TokenGroups, pGroups, dwSize, &dwSize))
                     {
-                        if (EqualSid (psidAdmin, pGroups->Groups[ iGroup ].Sid))
-                            fAdmin = TRUE;
+                        /* Look through the list of group SIDs and see if any of them
+                         * matches the AFS Client Admin group SID.
+                         */
+                        size_t iGroup = 0;
+                        for (; (!fAdmin) && (iGroup < pGroups->GroupCount); ++iGroup)
+                        {
+                            if (EqualSid (psidAdmin, pGroups->Groups[ iGroup ].Sid)) {
+                                fAdmin = TRUE;
+                            }
+                        }
                     }
+
+                    if (pGroups)
+                        free(pGroups);
                 }
 
-                if (pGroups)
-                    free(pGroups);
+                /* if do not have permission because we were not explicitly listed
+                 * in the Admin Client Group let's see if we are the SYSTEM account
+                 */
+                if (!fAdmin) {
+                    PTOKEN_USER pTokenUser;
+                    SID_IDENTIFIER_AUTHORITY SIDAuth = SECURITY_NT_AUTHORITY;
+                    PSID pSidLocalSystem = 0;
+                    DWORD gle;
+
+                    GetTokenInformation(hToken, TokenUser, NULL, 0, &dwSize);
+
+                    pTokenUser = (PTOKEN_USER)malloc(dwSize);
+
+                    if (!GetTokenInformation(hToken, TokenUser, pTokenUser, dwSize, &dwSize))
+                        gle = GetLastError();
+
+                    if (AllocateAndInitializeSid( &SIDAuth, 1,
+                                                  SECURITY_LOCAL_SYSTEM_RID,
+                                                  0, 0, 0, 0, 0, 0, 0,
+                                                  &pSidLocalSystem))
+                    {
+                        if (EqualSid(pTokenUser->User.Sid, pSidLocalSystem)) {
+                            fAdmin = TRUE;
+                        }
+
+                        FreeSid(pSidLocalSystem);
+                    }
+
+                    if ( pTokenUser )
+                        free(pTokenUser);
+                }
             }
         }
 
-        if (psidAdmin)
-            FreeSid (psidAdmin);
+        free(psidAdmin);
+        free(pszRefDomain);
+
+        fTested = TRUE;
     }
 
     return fAdmin;
@@ -1657,7 +1729,7 @@ register struct cmd_syndesc *as; {
     if ( checkserv.tinterval != 0 ) {
 #ifdef WIN32
         if ( !IsAdmin() ) {
-            fprintf (stderr,"Permission denied: requires Administrator access.\n");
+            fprintf (stderr,"Permission denied: requires AFS Client Administrator access.\n");
             return EACCES;
         }
 #else /* WIN32 */
@@ -1769,7 +1841,7 @@ register struct cmd_syndesc *as; {
     
 #ifdef WIN32
     if ( !IsAdmin() ) {
-        fprintf (stderr,"Permission denied: requires Administrator access.\n");
+        fprintf (stderr,"Permission denied: requires AFS Client Administrator access.\n");
         return EACCES;
     }
 #else /* WIN32 */
@@ -1883,7 +1955,7 @@ register struct cmd_syndesc *as; {
 
 #ifdef WIN32
     if ( !IsAdmin() ) {
-        fprintf (stderr,"Permission denied: requires Administrator access.\n");
+        fprintf (stderr,"Permission denied: requires AFS Client Administrator access.\n");
         return EACCES;
     }
 #else /* WIN32 */
@@ -2104,7 +2176,7 @@ register struct cmd_syndesc *as; {
     if (ti) {
 #ifdef WIN32
     if ( !IsAdmin() ) {
-        fprintf (stderr,"Permission denied: requires Administrator access.\n");
+        fprintf (stderr,"Permission denied: requires AFS Client Administrator access.\n");
         return EACCES;
     }
 #else /* WIN32 */
@@ -2144,19 +2216,19 @@ register struct cmd_syndesc *as; {
         return 0;
     }
 
-	input = space;
-	memcpy(&setp, input, sizeof(afs_int32));
-	input += sizeof(afs_int32);
-	if (!setp) {
-	    fprintf(stderr,"No sysname name value was found\n");
+    input = space;
+    memcpy(&setp, input, sizeof(afs_int32));
+    input += sizeof(afs_int32);
+    if (!setp) {
+        fprintf(stderr,"No sysname name value was found\n");
         return 1;
-	} 
+    } 
     
     printf("Current sysname%s", setp > 1 ? "s are" : " is");
     for (; setp > 0; --setp ) {
         printf(" \'%s\'", input);
         input += strlen(input) + 1;
-	}
+    }
     printf("\n");
     return 0;
 }
@@ -2171,7 +2243,7 @@ register struct cmd_syndesc *as; {
     
 #ifdef WIN32
     if ( !IsAdmin() ) {
-        fprintf (stderr,"Permission denied: requires Administrator access.\n");
+        fprintf (stderr,"Permission denied: requires AFS Client Administrator access.\n");
         return EACCES;
     }
 #else /* WIN32 */
@@ -2311,7 +2383,7 @@ register struct cmd_syndesc *as; {
 
 #ifdef WIN32
     if ( !IsAdmin() ) {
-        fprintf (stderr,"Permission denied: requires Administrator access.\n");
+        fprintf (stderr,"Permission denied: requires AFS Client Administrator access.\n");
         return EACCES;
     }
 #else /* WIN32 */
@@ -2410,7 +2482,7 @@ struct afsconf_cell *info;
 	}
 	else {
 	    /* got a ticket */
-	    if (ttoken.kvno >= 0 && ttoken.kvno	<= 255)	scIndex	= 2;	/* kerberos */
+	    if (ttoken.kvno >= 0 && ttoken.kvno	<= 256)	scIndex	= 2;	/* kerberos */
 	    else {
 		fprintf (stderr, "fs: funny kvno (%d) in ticket, proceeding\n",
 			 ttoken.kvno);
@@ -2567,7 +2639,7 @@ register struct cmd_syndesc *as; {
 
 #ifdef WIN32
     if ( !IsAdmin() ) {
-        fprintf (stderr,"Permission denied: requires Administrator access.\n");
+        fprintf (stderr,"Permission denied: requires AFS Client Administrator access.\n");
         return EACCES;
     }
 #else /* WIN32 */
@@ -2724,7 +2796,7 @@ static TraceCmd(struct cmd_syndesc *asp)
     
 #ifdef WIN32
     if ( !IsAdmin() ) {
-        fprintf (stderr,"Permission denied: requires Administrator access.\n");
+        fprintf (stderr,"Permission denied: requires AFS Client Administrator access.\n");
         return EACCES;
     }
 #else /* WIN32 */
@@ -2780,7 +2852,7 @@ struct cmd_syndesc *as; {
     
 #ifdef WIN32
     if ( !IsAdmin() ) {
-        fprintf (stderr,"Permission denied: requires Administrator access.\n");
+        fprintf (stderr,"Permission denied: requires AFS Client Administrator access.\n");  
         return EACCES;
     }
 #else /* WIN32 */
@@ -2845,7 +2917,7 @@ static afs_int32 SetCryptCmd(as)
  
 #ifdef WIN32
     if ( !IsAdmin() ) {
-        fprintf (stderr,"Permission denied: requires Administrator access.\n");
+        fprintf (stderr,"Permission denied: requires AFS Client Administrator access.\n");
         return EACCES;
     }
 #else /* WIN32 */
@@ -2906,7 +2978,7 @@ int argc;
 char **argv; {
     register afs_int32 code;
     register struct cmd_syndesc *ts;
-    
+
 #ifdef	AFS_AIX32_ENV
     /*
      * The following signal action for AIX is necessary so that in case of a 
@@ -3104,7 +3176,7 @@ char **argv; {
     cmd_AddParm(ts, "-path", CMD_LIST, CMD_OPTIONAL, "dir/file path");
 
     ts = cmd_CreateSyntax("sysname", SysNameCmd, 0, "get/set sysname (i.e. @sys) value");
-    cmd_AddParm(ts, "-newsys", CMD_SINGLE, CMD_OPTIONAL, "new sysname");
+    cmd_AddParm(ts, "-newsys", CMD_LIST, CMD_OPTIONAL, "new sysname");
 
     ts = cmd_CreateSyntax("exportafs", ExportAfsCmd, 0, "enable/disable translators to AFS");
     cmd_AddParm(ts, "-type", CMD_SINGLE, 0, "exporter name");
@@ -3239,68 +3311,106 @@ static MemDumpCmd(struct cmd_syndesc *asp)
 
 static CSCPolicyCmd(struct cmd_syndesc *asp)
 {
-	struct cmd_item *ti;
-	char *share = NULL;
-	char sbmtpath[256];
-	char *policy;
-	
-	for(ti=asp->parms[0].items; ti;ti=ti->next) {
-		share = ti->data;
-		if (share)
-		{
-			break;
-		}
-	}
-	
-	if (share)
-	{
-#ifdef WIN32
-        if ( !IsAdmin() ) {
+    struct cmd_item *ti;
+    char *share = NULL;
+    HKEY hkCSCPolicy;
+
+    for(ti=asp->parms[0].items; ti;ti=ti->next) {
+        share = ti->data;
+        if (share)
+        {
+            break;
+        }
+    }
+
+    if (share)
+    {
+        char *policy;
+
+        RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
+                        "SOFTWARE\\OpenAFS\\Client\\CSCPolicy",
+                        0, 
+                        "AFS", 
+                        REG_OPTION_NON_VOLATILE,
+                        KEY_WRITE,
+                        NULL, 
+                        &hkCSCPolicy,
+                        NULL );
+
+        if ( hkCSCPolicy == NULL ) {
             fprintf (stderr,"Permission denied: requires Administrator access.\n");
             return EACCES;
         }
-#else /* WIN32 */
-        if (geteuid()) {
-            fprintf (stderr,"Permission denied: requires root access.\n");
+
+        if ( !IsAdmin() ) {
+            fprintf (stderr,"Permission denied: requires AFS Client Administrator access.\n");
+            RegCloseKey(hkCSCPolicy);
             return EACCES;
         }
-#endif /* WIN32 */
 
         policy = "manual";
 		
-		if (asp->parms[1].items)
-			policy = "manual";
-		if (asp->parms[2].items)
-			policy = "programs";
-		if (asp->parms[3].items)
-			policy = "documents";
-		if (asp->parms[4].items)
-			policy = "disable";
+        if (asp->parms[1].items)
+            policy = "manual";
+        if (asp->parms[2].items)
+            policy = "programs";
+        if (asp->parms[3].items)
+            policy = "documents";
+        if (asp->parms[4].items)
+            policy = "disable";
 		
-		strcpy(sbmtpath, "afsdsbmt.ini");
-		WritePrivateProfileString("CSC Policy", share, policy, sbmtpath);
+        RegSetValueEx( hkCSCPolicy, share, 0, REG_SZ, policy, strlen(policy)+1);
 		
-		printf("CSC policy on share \"%s\" changed to \"%s\".\n\n", share, policy);
-		printf("Close all applications that accessed files on this share or restart AFS Client for the change to take effect.\n"); 
-	}
-	else
-	{
-		char policies[1024];
-		DWORD len = sizeof(policies);
+        printf("CSC policy on share \"%s\" changed to \"%s\".\n\n", share, policy);
+        printf("Close all applications that accessed files on this share or restart AFS Client for the change to take effect.\n"); 
+    }
+    else
+    {
+        DWORD dwIndex, dwPolicies;
+        char policyName[256];
+        DWORD policyNameLen;
+        char policy[256];
+        DWORD policyLen;
+        DWORD dwType;
 
-		/* list current csc policies */
-		strcpy(sbmtpath, "afsdsbmt.ini");
-				
-		GetPrivateProfileSection("CSC Policy", policies, len, sbmtpath);
-		
-		printf("Current CSC policies:\n");
-		policy = policies;
-		while (policy[0])
-		{
-			printf("  %s\n", policy);
-			policy += strlen(policy) + 1;
-		}
-	}
+        /* list current csc policies */
 
-	return (0);
+        RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
+                        "SOFTWARE\\OpenAFS\\Client\\CSCPolicy",
+                        0, 
+                        "AFS", 
+                        REG_OPTION_NON_VOLATILE,
+                        KEY_READ|KEY_QUERY_VALUE,
+                        NULL, 
+                        &hkCSCPolicy,
+                        NULL );
+
+        RegQueryInfoKey( hkCSCPolicy,
+                         NULL,  /* lpClass */
+                         NULL,  /* lpcClass */
+                         NULL,  /* lpReserved */
+                         NULL,  /* lpcSubKeys */
+                         NULL,  /* lpcMaxSubKeyLen */
+                         NULL,  /* lpcMaxClassLen */
+                         &dwPolicies, /* lpcValues */
+                         NULL,  /* lpcMaxValueNameLen */
+                         NULL,  /* lpcMaxValueLen */
+                         NULL,  /* lpcbSecurityDescriptor */
+                         NULL   /* lpftLastWriteTime */
+                         );
+		
+        printf("Current CSC policies:\n");
+        for ( dwIndex = 0; dwIndex < dwPolicies; dwIndex ++ ) {
+
+            policyNameLen = sizeof(policyName);
+            policyLen = sizeof(policy);
+            RegEnumValue( hkCSCPolicy, dwIndex, policyName, &policyNameLen, NULL,
+                          &dwType, policy, &policyLen);
+
+            printf("  %s = %s\n", policyName, policy);
+        }
+    }
+
+    RegCloseKey(hkCSCPolicy);
+    return (0);
 }
