@@ -14,9 +14,10 @@
  */
 
 #include <afsconfig.h>
-#include "../afs/param.h"
+#include "afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/rx/IRIX/rx_kmutex.c,v 1.1 2002/02/01 20:30:30 kolya Exp $");
+RCSID
+    ("$Header: /cvs/openafs/src/rx/IRIX/rx_kmutex.c,v 1.3 2003/07/15 23:16:21 shadow Exp $");
 
 /*
  * Currently everything is implemented in rx_kmutex.h
